package com.example.autoservice.components.api;

import java.util.List;

import org.modelmapper.ModelMapper;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import com.example.autoservice.components.model.ComponentsEntity;
import com.example.autoservice.components.service.ComponentsService;
import com.example.autoservice.core.configuration.Constants;

import jakarta.validation.Valid;

@Controller
@RequestMapping(ComponentsController.URL)
public class ComponentsController {
    public static final String URL = "/components";
    private static final String COMPONENTS_VIEW = "components";
    private static final String COMPONENTS_ATTRIBUTE = "components";
    private static final String COMPONENTS_EDIT_VIEW = "component-edit";
    private final ComponentsService componentsService;
    private final ModelMapper modelMapper;

    public ComponentsController(ComponentsService componentsService, ModelMapper modelMapper) {
        this.componentsService = componentsService;
        this.modelMapper = modelMapper;
    }

    private ComponentsDto toDto(ComponentsEntity entity) {
        return modelMapper.map(entity, ComponentsDto.class);
    }

    private ComponentsEntity toEntity(ComponentsDto dto) {
        return modelMapper.map(dto, ComponentsEntity.class);
    }
    @GetMapping
    public String getAll(Model model){
        List<ComponentsDto> components = componentsService.getAll().stream()
        .map(this::toDto)
        .toList();
        model.addAttribute(COMPONENTS_ATTRIBUTE,
            components);
        return COMPONENTS_VIEW;
    }

    @GetMapping("/edit/")
    public String create(Model model){
        model.addAttribute(COMPONENTS_ATTRIBUTE, new ComponentsDto());
        return COMPONENTS_EDIT_VIEW;
    }

    @PostMapping("/edit/")
    public String create(
            @ModelAttribute(name = COMPONENTS_ATTRIBUTE) @Valid ComponentsDto component,
            BindingResult bindingResult,
            Model model) {
        if (bindingResult.hasErrors()) {
            return COMPONENTS_EDIT_VIEW;
        }
        componentsService.create(toEntity(component));
        return Constants.REDIRECT_VIEW + URL;
    }

    @GetMapping("/edit/{id}")
    public String update(
            @PathVariable(name = "id") Long id,
            Model model) {
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        model.addAttribute(COMPONENTS_ATTRIBUTE, toDto(componentsService.get(id)));
        return COMPONENTS_EDIT_VIEW;
    }

    @PostMapping("/edit/{id}")
    public String update(
            @PathVariable(name = "id") Long id,
            @ModelAttribute(name = COMPONENTS_ATTRIBUTE) @Valid ComponentsDto component,
            BindingResult bindingResult,
            Model model) {
        if (bindingResult.hasErrors()) {
            return COMPONENTS_EDIT_VIEW;
        }
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        componentsService.update(id, toEntity(component));
        return Constants.REDIRECT_VIEW + URL;
    }

    @PostMapping("/delete/{id}")
    public String delete(
            @PathVariable(name = "id") Long id) {
        componentsService.delete(id);
        return Constants.REDIRECT_VIEW + URL;
    }
}
