package com.example.autoservice.components.service;

import java.util.List;
import java.util.stream.StreamSupport;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.example.autoservice.components.model.ComponentsEntity;
import com.example.autoservice.components.repository.ComponentsRepository;
import com.example.autoservice.core.error.NotFoundException;

@Service
public class ComponentsService {
    private final ComponentsRepository repository;
    public ComponentsService(ComponentsRepository repository) {
        this.repository = repository;
    }

    @Transactional(readOnly = true)
    public List<ComponentsEntity> getAll() {
        return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
    }

    @Transactional(readOnly = true)
    public ComponentsEntity get(Long id) {
        return repository.findById(id)
                .orElseThrow(() -> new NotFoundException(ComponentsEntity.class, id));
    }

    @Transactional
    public ComponentsEntity create(ComponentsEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }
        return repository.save(entity);
    }

    @Transactional
    public ComponentsEntity update(Long id, ComponentsEntity entity) {
        final ComponentsEntity existsEntity = get(id);
        existsEntity.setName(entity.getName());
        existsEntity.setPrice(entity.getPrice());
        return repository.save(existsEntity);
    }

    @Transactional
    public ComponentsEntity delete(Long id) {
        final ComponentsEntity existsEntity = get(id);
        repository.delete(existsEntity);
        return existsEntity;
    }
}
