package com.example.autoservice.services.api;

import java.util.List;

import org.modelmapper.ModelMapper;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import com.example.autoservice.services.model.ServicesEntity;
import com.example.autoservice.services.service.ServicesService;
import com.example.autoservice.core.configuration.Constants;

import jakarta.validation.Valid;

@Controller
@RequestMapping(ServicesController.URL)
public class ServicesController {
    public static final String URL = "/services";
    private static final String SERVICES_VIEW = "services";
    private static final String SERVICES_ATTRIBUTE = "services";
    private static final String SERVICES_EDIT_VIEW = "service-edit";
    private final ServicesService servicesService;
    private final ModelMapper modelMapper;

    public ServicesController(ServicesService servicesService, ModelMapper modelMapper) {
        this.servicesService = servicesService;
        this.modelMapper = modelMapper;
    }

    private ServicesDto toDto(ServicesEntity entity) {
        return modelMapper.map(entity, ServicesDto.class);
    }

    private ServicesEntity toEntity(ServicesDto dto) {
        return modelMapper.map(dto, ServicesEntity.class);
    }
    @GetMapping
    public String getAll(Model model){
        List<ServicesDto> services = servicesService.getAll().stream()
        .map(this::toDto)
        .toList();
        model.addAttribute(SERVICES_ATTRIBUTE,
            services);
        return SERVICES_VIEW;
    }

    @GetMapping("/edit/")
    public String create(Model model){
        model.addAttribute(SERVICES_ATTRIBUTE, new ServicesDto());
        return SERVICES_EDIT_VIEW;
    }

    @PostMapping("/edit/")
    public String create(
            @ModelAttribute(name = SERVICES_ATTRIBUTE) @Valid ServicesDto service,
            BindingResult bindingResult,
            Model model) {
        if (bindingResult.hasErrors()) {
            return SERVICES_EDIT_VIEW;
        }
        servicesService.create(toEntity(service));
        return Constants.REDIRECT_VIEW + URL;
    }

    @GetMapping("/edit/{id}")
    public String update(
            @PathVariable(name = "id") Long id,
            Model model) {
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        model.addAttribute(SERVICES_ATTRIBUTE, toDto(servicesService.get(id)));
        return SERVICES_EDIT_VIEW;
    }

    @PostMapping("/edit/{id}")
    public String update(
            @PathVariable(name = "id") Long id,
            @ModelAttribute(name = SERVICES_ATTRIBUTE) @Valid ServicesDto service,
            BindingResult bindingResult,
            Model model) {
        if (bindingResult.hasErrors()) {
            return SERVICES_EDIT_VIEW;
        }
        if (id <= 0) {
            throw new IllegalArgumentException();
        }
        servicesService.update(id, toEntity(service));
        return Constants.REDIRECT_VIEW + URL;
    }

    @PostMapping("/delete/{id}")
    public String delete(
            @PathVariable(name = "id") Long id) {
        servicesService.delete(id);
        return Constants.REDIRECT_VIEW + URL;
    }
}
