package com.example.autoservice.services.service;

import java.util.List;
import java.util.stream.StreamSupport;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.example.autoservice.services.model.ServicesEntity;
import com.example.autoservice.services.repository.ServicesRepository;
import com.example.autoservice.core.error.NotFoundException;

@Service
public class ServicesService {
    private final ServicesRepository repository;
    public ServicesService(ServicesRepository repository) {
        this.repository = repository;
    }

    @Transactional(readOnly = true)
    public List<ServicesEntity> getAll() {
        return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
    }

    @Transactional(readOnly = true)
    public ServicesEntity get(Long id) {
        return repository.findById(id)
                .orElseThrow(() -> new NotFoundException(ServicesEntity.class, id));
    }

    @Transactional
    public ServicesEntity create(ServicesEntity entity) {
        if (entity == null) {
            throw new IllegalArgumentException("Entity is null");
        }
        return repository.save(entity);
    }

    @Transactional
    public ServicesEntity update(Long id, ServicesEntity entity) {
        final ServicesEntity existsEntity = get(id);
        existsEntity.setName(entity.getName());
        existsEntity.setPrice(entity.getPrice());
        return repository.save(existsEntity);
    }

    @Transactional
    public ServicesEntity delete(Long id) {
        final ServicesEntity existsEntity = get(id);
        repository.delete(existsEntity);
        return existsEntity;
    }
}
